@@ -3,10 +3,11 @@ from django.contrib.auth.decorators import login_required
 from django.contrib.sites.shortcuts import get_current_site
 from django.core.exceptions import PermissionDenied
 from django.core.validators import validate_email
-from django.forms import ValidationError
+from django.forms import Form, ValidationError
 from django.http import Http404, HttpResponse, HttpResponseRedirect
 from django.urls import reverse, reverse_lazy
 from django.utils.decorators import method_decorator
+from django.utils.functional import cached_property
 from django.views.decorators.cache import never_cache
 from django.views.decorators.debug import sensitive_post_parameters
 from django.views.generic.base import TemplateView
@@ -17,6 +18,7 @@ from allauth.account import app_settings
 from allauth.account.adapter import get_adapter
 from allauth.account.forms import (
     AddEmailForm,
+    ChangeEmailForm,
     ChangePasswordForm,
     ChangePhoneForm,
     ConfirmEmailVerificationCodeForm,
@@ -33,9 +35,9 @@ from allauth.account.forms import (
     VerifyPhoneForm,
 )
 from allauth.account.internal import flows
-from allauth.account.internal.decorators import (
-    login_not_required,
-    login_stage_required,
+from allauth.account.internal.decorators import login_not_required, login_stage_required
+from allauth.account.internal.flows.email_verification import (
+    send_verification_email_to_address,
 )
 from allauth.account.mixins import (
     AjaxCapableProcessFormViewMixin,
@@ -56,14 +58,11 @@ from allauth.account.stages import (
     LoginStageController,
     PhoneVerificationStage,
 )
-from allauth.account.utils import (
-    send_email_confirmation,
-    sync_user_email_addresses,
-    user_display,
-)
+from allauth.account.utils import user_display
 from allauth.core import ratelimit
 from allauth.core.exceptions import ImmediateHttpResponse
 from allauth.core.internal.httpkit import redirect
+from allauth.core.ratelimit import RateLimited
 from allauth.decorators import rate_limit
 from allauth.utils import get_form_class
 
@@ -358,7 +357,7 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
 
     def dispatch(self, request, *args, **kwargs):
         self._did_send_verification_email = False
-        sync_user_email_addresses(request.user)
+        flows.manage_email.sync_user_email_address(request.user)
         return super().dispatch(request, *args, **kwargs)
 
     def get_form_kwargs(self):
@@ -406,12 +405,16 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
 
     def _action_send(self, request, *args, **kwargs):
         email_address = self._get_email_address(request)
+        did_send_verification_email = False
         if email_address:
-            send_email_confirmation(
-                self.request, request.user, email=email_address.email
+            did_send_verification_email = send_verification_email_to_address(
+                self.request, email_address
             )
-            self._did_send_verification_email = True
-        if app_settings.EMAIL_VERIFICATION_BY_CODE_ENABLED:
+        self._did_send_verification_email = did_send_verification_email
+        if (
+            app_settings.EMAIL_VERIFICATION_BY_CODE_ENABLED
+            and did_send_verification_email
+        ):
             return HttpResponseRedirect(reverse("account_email_verification_sent"))
 
     def _action_remove(self, request, *args, **kwargs):
@@ -795,6 +798,7 @@ class ConfirmPasswordResetCodeView(NextRedirectMixin, FormView):
     def get_context_data(self, **kwargs):
         ret = super().get_context_data(**kwargs)
         ret["email"] = self._process.state["email"]
+        ret["verify_form"] = ret["form"]
         return ret
 
     def form_valid(self, form):
@@ -860,7 +864,7 @@ class EmailVerificationSentView(TemplateView):
     template_name = "account/verification_sent." + app_settings.TEMPLATE_EXTENSION
 
 
-class ConfirmEmailVerificationCodeView(FormView):
+class ConfirmEmailVerificationCodeView(NextRedirectMixin, FormView):
     template_name = (
         "account/confirm_email_verification_code." + app_settings.TEMPLATE_EXTENSION
     )
@@ -890,29 +894,94 @@ class ConfirmEmailVerificationCodeView(FormView):
             )
         return super().dispatch(request, *args, **kwargs)
 
+    @cached_property
+    def _action(self):
+        action = self.request.POST.get("action")
+        valid_actions = ["verify"]
+        if self._process.can_change:
+            valid_actions.append("change")
+        if self._process.can_resend:
+            valid_actions.append("resend")
+        if action not in valid_actions:
+            action = "verify"
+        return action
+
     def get_form_class(self):
+        if self._action == "change":
+            return self._get_change_form_class()
+        elif self._action == "resend":
+            return Form
+        return self._get_verify_form_class()
+
+    def _get_change_form_class(self):
+        return ChangeEmailForm
+
+    def _get_verify_form_class(self):
         return get_form_class(
             app_settings.FORMS, "confirm_email_verification_code", self.form_class
         )
 
     def get_form_kwargs(self):
         ret = super().get_form_kwargs()
-        ret["code"] = self._process.code if self._process else ""
+        if self._action == "change":
+            pass
+        elif self._action == "verify":
+            ret["code"] = self._process.code if self._process else ""
+            ret["user"] = self._process.user
+            ret["email"] = self._process.email
         return ret
 
     def get_context_data(self, **kwargs):
         ret = super().get_context_data(**kwargs)
+        ret["can_change"] = self._process.can_change
+        ret["can_resend"] = self._process.can_resend
         ret["email"] = self._process.state["email"]
         ret["cancel_url"] = None if self.stage else reverse("account_email")
+        if self._action == "change":
+            ret["change_form"] = ret["form"]
+            ret["verify_form"] = self._get_verify_form_class()()
+        else:
+            ret["change_form"] = self._get_change_form_class()()
+            ret["verify_form"] = ret["form"]
         return ret
 
     def form_valid(self, form):
+        if self._action == "change":
+            return self._change_form_valid(form)
+        elif self._action == "resend":
+            return self._resend_form_valid(form)
+        return self._verify_form_valid(form)
+
+    def _resend_form_valid(self, form):
+        adapter = get_adapter()
+        try:
+            self._process.resend()
+        except RateLimited:
+            adapter.add_message(
+                self.request,
+                messages.ERROR,
+                message=adapter.error_messages["rate_limited"],
+            )
+        return HttpResponseRedirect(
+            self.passthrough_next_url(reverse("account_email_verification_sent"))
+        )
+
+    def _change_form_valid(self, form):
+        self._process.change_to(form.cleaned_data["email"], form.account_already_exists)
+        return HttpResponseRedirect(
+            self.passthrough_next_url(reverse("account_email_verification_sent"))
+        )
+
+    def _verify_form_valid(self, form):
         email_address = self._process.finish()
         if self.stage:
             if not email_address:
                 return self.stage.abort()
             return self.stage.exit()
-        if not email_address:
+        url = self.get_next_url()
+        if url:
+            pass
+        elif not email_address:
             url = reverse("account_email")
         else:
             url = get_adapter(self.request).get_email_verification_redirect_url(
@@ -921,6 +990,11 @@ class ConfirmEmailVerificationCodeView(FormView):
         return HttpResponseRedirect(url)
 
     def form_invalid(self, form):
+        if self._action == "change":
+            return self._change_form_invalid(form)
+        return self._verify_form_invalid(form)
+
+    def _verify_form_invalid(self, form):
         attempts_left = self._process.record_invalid_attempt()
         if attempts_left:
             return super().form_invalid(form)
@@ -1125,6 +1199,7 @@ class ConfirmLoginCodeView(NextRedirectMixin, FormView):
                 "site": site,
                 "email": email,
                 "phone": phone,
+                "verify_form": ret["form"],
             }
         )
         return ret
@@ -1139,19 +1214,84 @@ class _BaseVerifyPhoneView(NextRedirectMixin, FormView):
         "account/confirm_phone_verification_code." + app_settings.TEMPLATE_EXTENSION
     )
 
+    @cached_property
+    def _action(self):
+        action = self.request.POST.get("action")
+        valid_actions = ["verify"]
+        if self.process.can_change:
+            valid_actions.append("change")
+        if self.process.can_resend:
+            valid_actions.append("resend")
+        if action not in valid_actions:
+            action = "verify"
+        return action
+
     def get_form_class(self):
+        if self._action == "change":
+            return self._get_change_form_class()
+        elif self._action == "resend":
+            return Form
+        return self._get_verify_form_class()
+
+    def _get_change_form_class(self):
+        return get_form_class(app_settings.FORMS, "change_phone", ChangePhoneForm)
+
+    def _get_verify_form_class(self):
         return get_form_class(app_settings.FORMS, "verify_phone", self.form_class)
 
     def get_form_kwargs(self):
         kwargs = super().get_form_kwargs()
-        kwargs["code"] = self.process.code
+        if self._action == "change":
+            kwargs["phone"] = self.process.phone
+            kwargs["user"] = self.process.user
+        elif self._action == "resend":
+            pass
+        else:
+            kwargs["code"] = self.process.code
+            kwargs["phone"] = self.process.phone
+            kwargs["user"] = self.process.user
         return kwargs
 
     def form_valid(self, form):
+        if self._action == "change":
+            return self._change_form_valid(form)
+        elif self._action == "resend":
+            return self._resend_form_valid(form)
+        return self._verify_form_valid(form)
+
+    def _resend_form_valid(self, form):
+        try:
+            self.process.resend()
+        except RateLimited:
+            adapter = get_adapter()
+            adapter.add_message(
+                self.request,
+                messages.ERROR,
+                message=adapter.error_messages["rate_limited"],
+            )
+        return HttpResponseRedirect(
+            self.passthrough_next_url(reverse("account_verify_phone"))
+        )
+
+    def _change_form_valid(self, form):
+        self.process.change_to(form.cleaned_data["phone"], form.account_already_exists)
+        return HttpResponseRedirect(
+            self.passthrough_next_url(reverse("account_verify_phone"))
+        )
+
+    def _verify_form_valid(self, form):
         self.process.finish()
         return self.respond_process_succeeded(form)
 
     def form_invalid(self, form):
+        if self._action == "change":
+            return self._change_form_invalid(form)
+        return self._verify_form_invalid(form)
+
+    def _change_form_invalid(self, form):
+        return super().form_invalid(form)
+
+    def _verify_form_invalid(self, form):
         attempts_left = self.process.record_invalid_attempt()
         if attempts_left:
             return super().form_invalid(form)
@@ -1160,11 +1300,20 @@ class _BaseVerifyPhoneView(NextRedirectMixin, FormView):
 
     def get_context_data(self, **kwargs):
         ret = super().get_context_data(**kwargs)
+        ret["can_change"] = self.process.can_change
+        ret["can_resend"] = self.process.can_resend
         site = get_current_site(self.request)
+        if self._action == "change":
+            ret["change_form"] = ret["form"]
+            ret["verify_form"] = self._get_verify_form_class()()
+        else:
+            ret["change_form"] = self._get_change_form_class()()
+            ret["verify_form"] = ret["form"]
         ret.update(
             {
                 "site": site,
                 "phone": self.process.phone,
+                "action": self._action,
             }
         )
         return ret
@@ -1253,6 +1402,7 @@ class ChangePhoneView(FormView):
             ret["phone"] = None
         else:
             ret["phone"] = self._phone_verified[0] if self._phone_verified else None
+        ret["user"] = self.request.user
         return ret
 
     def form_valid(self, form):
