@@ -1,6 +1,5 @@
 from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns
-
-from .provider import StravaProvider
+from allauth.socialaccount.providers.strava.provider import StravaProvider
 
 
 urlpatterns = default_urlpatterns(StravaProvider)
