@@ -1,6 +1,5 @@
 from allauth.socialaccount.providers.oauth.urls import default_urlpatterns
-
-from .provider import TwitterProvider
+from allauth.socialaccount.providers.twitter.provider import TwitterProvider
 
 
 urlpatterns = default_urlpatterns(TwitterProvider)
