@@ -1,7 +1,4 @@
-from allauth.socialaccount.providers.base import (
-    ProviderAccount,
-    ProviderException,
-)
+from allauth.socialaccount.providers.base import ProviderAccount, ProviderException
 from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider
 from allauth.socialaccount.providers.weibo.views import WeiboOAuth2Adapter
 
@@ -14,10 +11,6 @@ class WeiboAccount(ProviderAccount):
     def get_avatar_url(self):
         return self.account.extra_data.get("avatar_large")
 
-    def to_str(self):
-        dflt = super(WeiboAccount, self).to_str()
-        return self.account.extra_data.get("name", dflt)
-
 
 class WeiboProvider(OAuth2Provider):
     id = "weibo"
